@@ -397,12 +397,15 @@ class EntryIDType(enum.Enum):
         """
         return EntryIDTypeHex[self.name]
 
+    # This is the same as the one used for permanent IDs, and the strucutre is
+    # near identical too. Anything that needs a PermanentEntryID will need to
+    # specifically ask for it instead of autogenerating it.
     ADDRESS_BOOK_RECIPIENT = b'\xDC\xA7\x40\xC8\xC0\x42\x10\x1A\xB4\xB9\x08\x00\x2B\x2F\xE1\x82'
     # Contact address or personal distribution list recipient.
     CA_OR_PDL_RECIPIENT = b'\xFE\x42\xAA\x0A\x18\xC7\x1A\x10\xE8\x85\x0B\x65\x1C\x24\x00\x00'
+    # This is also used for the Store Object EntryID structure.
     NNTP_NEWSGROUP_FOLDER = b'\x38\xA1\xBB\x10\x05\xE5\x10\x1A\xA1\xBB\x08\x00\x2B\x2A\x56\xC2'
     ONE_OFF_RECIPIENT = b'\x81\x2B\x1F\xA4\xBE\xA3\x10\x19\x9D\x6E\x00\xDD\x01\x0F\x54\x02'
-    PERMANENT = b'\xDC\xA7\x40\xC8\xC0\x42\x10\x1A\xB4\xB9\x08\x00\x2B\x2F\xE1\x82'
     PUBLIC_MESSAGE_STORE = b'\x1A\x44\x73\x90\xAA\x66\x11\xCD\x9B\xC8\x00\xAA\x00\x2F\xC4\x5A'
     # [MS-OXOCNTC] WrappedEntryId Structure.
     WRAPPED = b'\xC0\x91\xAD\xD3\x51\x9D\xCF\x11\xA4\xA9\x00\xAA\x00\x47\xFA\xA4'
@@ -425,7 +428,6 @@ class EntryIDTypeHex(enum.Enum):
     CA_OR_PDL_RECIPIENT = 'FE42AA0A18C71A10E8850B651C240000'
     NNTP_NEWSGROUP_FOLDER = '38A1BB1005E5101AA1BB08002B2A56C2'
     ONE_OFF_RECIPIENT = '812B1FA4BEA310199D6E00DD010F5402'
-    PERMANENT = 'DCA740C8C042101AB4B908002B2FE182'
     PUBLIC_MESSAGE_STORE = '1A447390AA6611CD9BC800AA002FC45A'
     # [MS-OXOCNTC] WrappedEntryId Structure.
     WRAPPED = 'C091ADD3519DCF11A4A900AA0047FAA4'
